/*
 * Sixth 3D engine. Copyright ©2012-2019, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui;

public class TextPointer implements Comparable<TextPointer> {

    public int row;
    public int column;

    public TextPointer() {
        this(0, 0);
    }

    public TextPointer(final int row, final int column) {
        this.row = row;
        this.column = column;
    }

    public TextPointer(final TextPointer parent) {
        this(parent.row, parent.column);
    }

    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;

        return o instanceof TextPointer && compareTo((TextPointer) o) == 0;
    }

    @Override
    public int hashCode() {
        int result = row;
        result = 31 * result + column;
        return result;
    }

    @Override
    public int compareTo(final TextPointer textPointer) {

        if (textPointer.row > row)
            return -1;
        if (textPointer.row < row)
            return 1;

        return Integer.compare(column, textPointer.column);

    }

    public boolean isBetween(final TextPointer start, final TextPointer end) {

        if (start == null)
            return false;

        if (end == null)
            return false;

        TextPointer smaller;
        TextPointer bigger;

        if (end.compareTo(start) >= 0) {
            smaller = start;
            bigger = end;
        } else {
            smaller = end;
            bigger = start;
        }

        return (compareTo(smaller) >= 0) && (bigger.compareTo(this) > 0);

    }

}
