/*
 * Sixth 3D engine. Copyright ©2012-2019, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.gui.humaninput;

import eu.svjatoslav.sixth.e3d.gui.ViewPanel;

import java.util.Stack;

public class KeyboardFocusStack {

    private final ViewPanel viewPanel;
    private WorldNavigationUserInputTracker defaultInputHandler = new WorldNavigationUserInputTracker();
    private Stack<UserInputHandler> inputHandlers = new Stack<>();
    private UserInputHandler currentUserInputHandler;

    public KeyboardFocusStack(final ViewPanel viewPanel) {
        this.viewPanel = viewPanel;
        pushFocusOwner(defaultInputHandler);
    }

    public UserInputHandler getCurrentFocusOwner() {
        return currentUserInputHandler;
    }

    public void popFocusOwner() {
        if (currentUserInputHandler != null)
            currentUserInputHandler.focusLost(viewPanel);

        if (inputHandlers.isEmpty())
            return;

        currentUserInputHandler = inputHandlers.pop();
        currentUserInputHandler.focusReceived(viewPanel);
    }

    public boolean pushFocusOwner(final UserInputHandler newInputHandler) {
        boolean updateNeeded = false;

        if (currentUserInputHandler == newInputHandler)
            return false;

        if (currentUserInputHandler != null) {
            updateNeeded = currentUserInputHandler.focusLost(viewPanel);
            inputHandlers.push(currentUserInputHandler);
        }

        currentUserInputHandler = newInputHandler;
        updateNeeded |= currentUserInputHandler.focusReceived(viewPanel);

        return updateNeeded;
    }
}
