/*
 * Sixth 3D engine. Copyright ©2012-2019, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes;

import eu.svjatoslav.sixth.e3d.geometry.Point3D;
import eu.svjatoslav.sixth.e3d.gui.RenderingContext;
import eu.svjatoslav.sixth.e3d.math.GeometryCoordinate;
import eu.svjatoslav.sixth.e3d.math.TransformPipe;
import eu.svjatoslav.sixth.e3d.renderer.raster.RenderAggregator;

import java.util.concurrent.atomic.AtomicInteger;

public abstract class AbstractCoordinateShape extends AbstractShape {

    public static final AtomicInteger lastShapeId = new AtomicInteger();
    public final int shapeId;
    public final GeometryCoordinate[] coordinates;
    public double onScreenZ;

    public AbstractCoordinateShape(final int pointsCount) {
        coordinates = new GeometryCoordinate[pointsCount];
        for (int i = 0; i < pointsCount; i++)
            coordinates[i] = new GeometryCoordinate();

        shapeId = lastShapeId.getAndIncrement();
    }

    public AbstractCoordinateShape(final Point3D... locationPoints) {
        coordinates = new GeometryCoordinate[locationPoints.length];

        for (int i = 0; i < locationPoints.length; i++)
            coordinates[i] = new GeometryCoordinate(locationPoints[i]);

        shapeId = lastShapeId.getAndIncrement();
    }

    public double getZ() {
        return onScreenZ;
    }

    public abstract void paint(RenderingContext renderBuffer);

    @Override
    public void transform(final TransformPipe transforms,
                          final RenderAggregator aggregator,
                          final RenderingContext renderingContext) {

        double accumulatedZ = 0;
        boolean paint = true;

        for (final GeometryCoordinate geometryPoint : coordinates) {
            geometryPoint.transform(transforms, renderingContext);

            accumulatedZ += geometryPoint.transformedCoordinate.z;

            if (!geometryPoint.transformedCoordinate.isVisible())
                paint = false;
        }

        if (paint) {
            onScreenZ = accumulatedZ / coordinates.length;
            aggregator.queueShapeForRendering(this);
        }
    }

}
