/*
 * Sixth 3D engine. Copyright ©2012-2019, Svjatoslav Agejenko, svjatoslav@svjatoslav.eu
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 3 of the GNU Lesser General Public License
 * or later as published by the Free Software Foundation.
 *
 */

package eu.svjatoslav.sixth.e3d.renderer.raster.shapes.basic.solidpolygon;

import eu.svjatoslav.sixth.e3d.geometry.Point2D;

public class LineInterpolator implements Comparable<LineInterpolator> {

    Point2D p1;
    Point2D p2;
    private int height;
    private int width;
    private int absoluteHeight;

    @Override
    public boolean equals(final Object o) {
        if (o == null) return false;

        return o instanceof LineInterpolator && compareTo((LineInterpolator) o) == 0;
    }

    @Override
    public int compareTo(final LineInterpolator o) {
        if (absoluteHeight < o.absoluteHeight)
            return 1;
        if (absoluteHeight > o.absoluteHeight)
            return -1;

        return Integer.compare(o.width, width);

    }

    @Override
    public int hashCode() {
        int result = width;
        result = 31 * result + absoluteHeight;
        return result;
    }

    public boolean containsY(final int y) {

        if (p1.y <= p2.y) {
            if (y >= p1.y)
                if (y <= p2.y)
                    return true;
        } else if (y >= p2.y)
            if (y <= p1.y)
                return true;

        return false;
    }

    public int getX(final int y) {

        if (height == 0)
            return (int) (p2.x + p1.x) / 2;

        return (int) (p1.x + ((width * (y - p1.y)) / height));
    }

    public void setPoints(final Point2D p1, final Point2D p2) {
        this.p1 = p1;
        this.p2 = p2;
        height = (int) (p2.y - p1.y);
        width = (int) (p2.x - p1.x);

        absoluteHeight = Math.abs(height);
    }

}
